@@ -1,6 +1,6 @@
 /* The MIT License
 
-    Copyright (C) 2020-2021 Giulio Genovese
+    Copyright (C) 2020-2023 Giulio Genovese
 
     Author: Giulio Genovese <giulio.genovese@gmail.com>
 
@@ -39,6 +39,7 @@ typedef struct
 {
     htsFile *fh;        // output file handle
     char *fname;        // output file name
+    char *index_fn;
 }
 subset_t;
 
@@ -60,6 +61,7 @@ typedef struct
     char **hts_opts;
     int nhts_opts;
     bcf_hdr_t *hdr;
+    int write_index;
 }
 args_t;
 
@@ -95,6 +97,7 @@ static const char *usage_text(void)
         "   -x, --extra STRING              Output records not overlapping listed regions in separate file\n"
         "   -p, --prefix STRING             Prepend string to output VCF names\n"
         "       --hts-opts LIST             Low-level options to pass to HTSlib, e.g. block_size=32768\n"
+        "   -W, --write-index[=FMT]         Automatically index the output files [off]\n"
         "\n"
         "Examples:\n"
         "   # Scatter a VCF file by shards with 10000 variants each\n"
@@ -200,6 +203,9 @@ static void open_set(subset_t *set, args_t *args)
         if ( args->record_cmd_line ) bcf_hdr_append_version(args->hdr, args->argc, args->argv, "bcftools_plugin");
     }
     if ( bcf_hdr_write(set->fh, args->hdr)!=0 ) error("[%s] Error: cannot write the header to %s\n", __func__, args->str.s);
+    if ( init_index2(set->fh,args->hdr,args->str.s,&set->index_fn,
+                     args->write_index)<0 )
+        error("Error: failed to initialise index for %s\n",args->str.s);
 }
 
 static void init_data(args_t *args)
@@ -260,7 +266,17 @@ static void destroy_data(args_t *args)
     for (i=0; i<args->nsets; i++)
     {
         subset_t *set = &args->sets[i];
-        if (set->fname) {
+        if (set->fname)
+        {
+            if ( args->write_index )
+            {
+                if ( bcf_idx_save(set->fh)<0 )
+                {
+                    if ( hts_close(set->fh)!=0 ) error("Error: close failed .. %s\n", set->fname);
+                    error("Error: cannot write to index %s\n", set->index_fn);
+                }
+                free(set->index_fn);
+            }
             if ( hts_close(set->fh)!=0 ) error("Error: close failed .. %s\n", set->fname);
             free(set->fname);
         }
@@ -290,7 +306,12 @@ static void process(args_t *args)
         args->rec_cnt++;
         if (args->rec_cnt == args->nsites) {
           args->rec_cnt = 0;
+          int err = 0;
+          if ( args->write_index && bcf_idx_save(set->fh) < 0)
+              err = 1;
           if ( hts_close(set->fh)!=0 ) error("Error: close failed .. %s\n", set->fname);
+          if (err)
+              error("Error: cannot write to index %s\n", set->index_fn);
           free(set->fname);
           set->fname = NULL;
           args->chunk_cnt++;
@@ -338,11 +359,12 @@ int run(int argc, char **argv)
         {"extra",required_argument,NULL,'x'},
         {"prefix",required_argument,NULL,'p'},
         {"hts-opts",required_argument,NULL,5},
+        {"write-index",optional_argument,NULL,'W'},
         {NULL,0,NULL,0}
     };
     int c;
     char *tmp;
-    while ((c = getopt_long(argc, argv, "e:i:o:O:r:R:t:T:n:s:S:x:p:h?", loptions, NULL)) >= 0)
+    while ((c = getopt_long(argc, argv, "e:i:o:O:r:R:t:T:n:s:S:x:p:W::h?", loptions, NULL)) >= 0)
     {
         switch (c)
         {
@@ -395,6 +417,10 @@ int run(int argc, char **argv)
             case 'x': args->extra = optarg;  break;
             case 'p': args->prefix = optarg;  break;
             case  5 : args->hts_opts = hts_readlist(optarg, 0, &args->nhts_opts); break;
+            case 'W':
+                if (!(args->write_index = write_index_parse(optarg)))
+                    error("Unsupported index format '%s'\n", optarg);
+                break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
